/*********************************************************************************
Date: November 2010, revised Oct 2011, revised Jul 2012
Purpose: Bayesian logistic regression
	Compares Greenland 2007 IJE, Bayes procedures with SAS' new Bayes procs. 
Notes: In this analysis there is no intercept prior, so results differ slightly from 
		those in Greenland 2007
	The syntax produces table 1 in the paper.  However, syntax needs to be modified
		for SAS 9.3 which no longer allows OddsRatios to be output though the ODS system. 
		Instead, output ParameterEstimates and calculate 95% confidence intervals manually.
Created by: Sheena Sullivan 
*********************************************************************************/

%let workdir= C:\Users\sgsullivan\Dropbox\Bayes; *assign your own directory;
libname bayes "&workdir";
filename fm0 "&workdir\fm0.asc";
libname mymacs "&workdir\macros";
options mstored sasmstore=mymacs;
options nodate nonumber nocenter formdlim= " " ;

/*******************************************************************
Import the data and get maximum likelihood (ML) estimates 
********************************************************************/;
data fm0;
	infile fm0;
	input death nonwhite teenages nullip gestage isoimm abort hydram dyslab 
		placord nomonit twint ward prerupt malpres;
	dyslab=dyslab/3; *recode dyslab;
	run;
proc logistic data=fm0 descending;	 *ML estimates on actual data;
	title "Maximum likelihood estimates, death on 14 regressors";
	model death = nonwhite--malpres / clodds=pl;
	ods output OddsRatios=actualw cloddspl=actualpl; *nb: OddsRatios cannot be output in SAS 9.3;
	run;
proc logistic data=fm0 descending;	 *Firth correction;
	title "Maximum likelihood estimates, death on 14 regressors";
	title2 "With Firth correction";
	model death = nonwhite--malpres / firth clodds=pl;
	run;

/*******************************************************************
The data augmentation procedure (DAP)
********************************************************************/;
/*Add the DAP variables to the actual data*/;
data actual;
	set fm0;
	Const=1;
	M1=1;
	H=0;
	run;
/*Create the prior data table*/;
data prior;						   
	input death nonwhite teenages nullip gestage isoimm abort hydram dyslab placord nomonit twint ward prerupt malpres Const M1 H;
datalines;
4	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	8	-0.693147181
4	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	8	-0.693147181
4	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	8	-0.693147181
4	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	8	-1.386294361
4	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	8	-0.693147181
4	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	8	0
4	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	8	-1.386294361
4	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	8	-0.693147181
4	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	8	-0.693147181
4	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	8	-0.693147181
4	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	8	-1.386294361
4	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	8	-0.693147181
4	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	8	-0.693147181
4	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	8	-1.386294361
;
run;
proc logistic data=prior;
	title "Logistic regression to get back priors";
	title2 "Priors are N(2,0.5) or N(4,0.5)";
	model death/M1 = Const nonwhite--malpres/noint offset=H clodds=pl;
	run;

/*Using the unscaled, incremented, A=4.5 prior*/;
data prior1; *increment the prior;
	set prior;
	death=death+0.5;
	M1=2*death;
	run;
proc logistic data=prior1;	 *get back prior;
	title "Logistic regression to get back priors (incremented priors, no prior on intercept)";
	model death/M1 = Const nonwhite--malpres/noint offset=H clodds=pl;
	ods output OddsRatios=prior1ors;
	run;
data augdata1; *augment actual data;
	set actual prior1;
	run;
proc logistic data=augdata1; *get posterior estimates;
	title "Logistic regression to obtain posterior estimates using data augmentation, A=4.5";
	model death/M1 = Const nonwhite--malpres/noint offset=H clodds=pl;
	ods output OddsRatios=postaug1w cloddspl=postaug1pl;
	run;

/*Rescaled prior, A=400*/;
data prior2; *rescale the prior data;
	set prior;
	s=10;
	death=death*s*s;
	M1=M1*s*s;
	array covars {*} nonwhite teenages nullip gestage isoimm abort hydram dyslab 
		placord nomonit twint ward prerupt malpres Const H;
	do i=1 to dim(covars);
		covars(i)=covars(i)/s;
		end;
	drop s i;
	run;
proc logistic data=prior2; *get back prior;
	title "Logistic regression to get back prior estimates and limits";
	model death/M1 = Const nonwhite--malpres/noint offset=H;
	ods output OddsRatios=prior2ors;
run;
data augdata2;	*augment the actual data;		 
	set actual prior2;
	run;
proc logistic data=augdata2; *get posterior estimates;
	title "Bayesian logistic regression of death on 14 regressors using data augmentation";
	model death/M1 = const nonwhite--malpres/noint offset=H clodds=pl;
	ods output OddsRatios=postaug2w cloddspl=postaug2pl;
	run;

/*******************************************************************
Comparison with MCMC (SAS Bayes statement in GENMOD)
********************************************************************/;
/*Create dataset containing the priors */;
data prior3;  
	input _type_ $ Intercept nonwhite teenages nullip gestage isoimm abort hydram 
					dyslab placord nomonit twint ward prerupt malpres ;
	datalines;
Var  1e6 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5
Mean 0 0.693147181 0.693147181 0.693147181 1.386294361 0.693147181 0 1.386294361 0.693147181 0.693147181 0.693147181 1.386294361 0.693147181 0.693147181 1.386294361
;
run;
ods graphics on;
ods html gpath="&workdir\plots";
proc genmod data=fm0 descending; *get the posterior estimates;
	title "Bayesian logistic regression of death on 14 regressors using MCMC";
	model death = nonwhite--malpres / dist=binomial link=logit lrci waldci;
	bayes nmc=100000 coeffprior=normal(input= prior3) stats(percent=2.5 50 97.5) 
		/*seed included to allow replication; not advised for actual analyses: */ seed=1234 
		/*request plots and diagnostics to check the MCMC performance*/ diagnostic=(autocorr ess) plots=all;
	ods output PostSummaries=sum;
	run;	
data postmcmc; *exponentiate the coefficients;
	set sum;
	mcmcmean=exp(mean);
	mcmcll=exp(P2_5);
	mcmcmed=exp(P50);
	mcmcul=exp(P97_5);
	effect=parameter;
	if Parameter="Intercept" then effect="Const";
	run;
proc print data=postmcmc; 
	title 'Posterior mean, median and 95% limits from MCMC analysis';
	var parameter mcmcmean mcmcmed mcmcll mcmcul;
	run;

/*******************************************************************
Table 1
	- requires the rounding macro
********************************************************************/;
/*Get frequencies for column 2*/
data fm0;					   /*recodings*/
	set fm0;
*rescaling - these were already reverse coded, but are coded back for the deaths column in the table;
	if nomonit=0 then nomonitr=1;
	if nomonit=1 then nomonitr=0;
	if nullip=0 then nullipr=1;
	if nullip=1 then nullipr=0;
	run;
/*Deaths with Xj>0*/;
proc format;
	value nonzero	0 	= "0"
				0<-high	= "nonzero";
	run;
proc freq data=fm0; 
	title "Deaths with Xj>0";
	format _ALL_ nonzero.;
	where death=1;
	ods output onewayfreqs=deaths;
	run;
%macro delzero(invar,dsn);
	data &dsn;
		set &dsn;
		%let k=1;
		%let maincat=%scan(&invar,&k);
		%do %while("&maincat" NE "");
			if &maincat=0 then delete;
			%let k=%eval(&k+1);
			%let maincat=%scan(&invar,&k);
		%end;
		run;
	%mend;
%let invars=nonwhite teenages nullipr gestage isoimm abort hydram dyslab placord nomonitr twint ward prerupt malpres;
%delzero(&invars,deaths); run;
data deaths;
	set deaths;
	effect=substr(Table,7);
	deaths=Frequency;
	if effect in("death","nomonit","nullip") then delete; 
	keep effect deaths;
	run;
data deaths;
	set deaths;
	if effect="nomonitr" then effect="nomonit";
	if effect="nullipr" then effect="nullip";
	run;
proc print data=deaths;
	title "Deaths with Xj>0";
	run;
proc sort data=deaths; by effect; 
proc sort data=prior2ors; by effect;
proc sort data=actualw; by effect;
proc sort data=actualpl; by effect;
proc sort data=postaug1w; by effect;
proc sort data=postaug1pl; by effect;
proc sort data=postaug2w; by effect;
proc sort data=postaug2pl; by effect;
proc sort data=postmcmc; by effect; run;
data table1;
	merge 	deaths
			prior2ors(rename=(OddsRatioEst=priorOR LowerCL=priorLL UpperCL=priorUL)) 
			actualw(rename=(OddsRatioEst=actualwOR LowerCL=actualwLL UpperCL=actualwUL)) 
			actualpl(rename=(OddsRatioEst=actualplOR LowerCL=actualplLL UpperCL=actualplUL)) 
			postaug1w(rename=(OddsRatioEst=postaug1wOR LowerCL=postaug1wLL UpperCL=postaug1wUL)) 
			postaug1pl(rename=(OddsRatioEst=postaug1plOR LowerCL=postaug1plLL UpperCL=postaug1plUL)) 
			postaug2w(rename=(OddsRatioEst=postaug2wOR LowerCL=postaug2wLL UpperCL=postaug2wUL)) 
			postaug2pl(rename=(OddsRatioEst=postaug2plOR LowerCL=postaug2plLL UpperCL=postaug2plUL))
			postmcmc
			;
	by effect;

*decimal places are to 2 informative digits, regardless of decimal point;
*prior;
	%rounding1(priorOR,priorLL,priorUL,prior);
*actual;
	%rounding2(actualwOR,actualwLL,actualwUL,actualw);
	%rounding2(actualplOR,actualplLL,actualplUL,actualpl);
	actual = (trim(left(actualw))) || ' (' || trim(left(ll_actualpl)) || ',' || trim(left(ul_actualpl)) || ')';	
*posterior, data augmentation, no rescaling, A=4.5;
	%rounding2(postaug1wOR,postaug1wLL,postaug1wUL,postaug1w);
	%rounding2(postaug1plOR,postaug1plLL,postaug1plUL,postaug1pl);
	postaug1 = (trim(left(postaug1w))) || ' (' || trim(left(ll_postaug1pl)) || ',' || trim(left(ul_postaug1pl)) || ')';
*posterior, data augmentation;
	%rounding2(postaug2wOR,postaug2wLL,postaug2wUL,postaug2w);
	%rounding2(postaug2plOR,postaug2plLL,postaug2plUL,postaug2pl);
	postaug2 = (trim(left(postaug2w))) || ' (' || trim(left(ll_postaug2pl)) || ',' || trim(left(ul_postaug2pl)) || ')';
*posterior, MCMC;
	%rounding2(mcmcmed,mcmcll,mcmcul,postmcmc);
*set order;
	length regressor $30.;
	if effect = "Const" then do; regressor = "Const"; order=1; end;
	if effect = "nonwhite" then do; regressor = "Non-white"; order=2; end;
	if effect = "teenages" then do; regressor = "Early age"; order=3; end;
	if effect = "nullip" then do; regressor = "Nulliparity"; order=4; end;
	if effect = "gestage" then do; regressor = "Gestational age"; order=5; end;
	if effect = "isoimm" then do; regressor = "Isoimmunization "; order=6; end;
	if effect = "abort" then do; regressor = "Past abortion"; order=7; end;
	if effect = "hydram" then do; regressor = "Hydramnios"; order=8; end;
	if effect = "dyslab" then do; regressor = "Labour progress "; order=9; end;
	if effect = "placord" then do; regressor = "PCA"; order=10; end;
	if effect = "nomonit" then do; regressor = "No monitor"; order=11; end;
	if effect = "twint" then do; regressor = "Twin, triplet"; order=12; end;
	if effect = "ward" then do; regressor = "Public ward"; order=13; end;
	if effect = "prerupt" then do; regressor = "PROM"; order=14; end;
	if effect = "malpres" then do; regressor = "Malpresented"; order=15; end;

	label	regressor = "Regressor (Xj)a"
			deaths = "Deaths with Xj>0"
			prior = "Prior median (95% limits)"
			actual = "ML estimates, A=1 (95%WL)(95%PL)"
			postaug1 = "Approximate DAP posterior median, A=4.5 (95%WL)(95%PL)"
			postaug2 = "Approximate DAP posterior median, rescaled A=400 (95%WL)(95%PL)"
			postmcmc = "Approximate MCMC posterior median (95% limits)"
	run;
proc sort; by order; run;
ods rtf file="&workdir\table1.rtf";
proc print data=table1 noobs label;
	title "Table 1. Prior, actual and posterior estimates";
	var regressor deaths prior actual postaug1 postaug2 postmcmc;
	run;
ods rtf close;
ods listing;
